/*
Copyright 2020 Peter Howkins

Permission is hereby granted, free of charge, to any person obtaining a
copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including
without limitation the rights to use, copy, modify, merge, publish,
distribute, sublicense, and/or sell copies of the Software, and to
permit persons to whom the Software is furnished to do so, subject to
the following conditions:

The above copyright notice and this permission notice shall be included
in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/
#ifndef BASIC____H
#define BASIC____H

/**
 * @file
 * @brief BBC BASIC style API
 * 
 * C wrappers in the style of BBC BASIC keywords
 */ 

#include <string.h>
#include <stdio.h>
#include "stdint.h"
#include "os.h"

/**
 * Clear the text area of the screen
 */ 
static inline void
CLS(void)
{
	oswrch(12);
}

/**
 * Set the text foreground and background colour
 *
 * @param colour 0-15 for foreground, 128-143 for background
 */ 
extern void COLOUR(uint8_t colour);

/**
 * PLOT is the multi-purpose point, line and triangle drawing statement in
 * BASIC. With the graphcis extension ROM fitted, it also provides shape and
 * sprite plotting routines.
 *
 * It is also the code behind the DRAW and MOVE keywords
 *
 * @param plotcode action to perform
 * @param x x coordinate in osunits
 * @param y y coordinate in osunits
 */
extern void PLOT(uint8_t plotcode, uint16_t x, uint16_t y);


/**
 * Draws lines on the screen in MODEs 0,1,2,4 and 5
 * draws from current graphics point set by MOVE or DRAW previously.
 * 
 * @param x end of line x coordinate in osunits
 * @param y end of line y coordinate in osunits
 */
static inline void
DRAW(uint16_t x, uint16_t y)
{
	PLOT(5, x, y);
}

/**
 * The envelope statement is used with the SOUND statement to control
 * the volume and pitch of a sound while it is playing.
 * 
 * @param block pointer to 14 byte block containing envelope parameters
 */ 
static inline void
ENVELOPE(const void *block)
{
	osword(8, block);
}

/**
 * Selects the graphics foreground colour and the graphics background 
 * colour.
 * 
 * @param action mode of action to use, 0 = plot, 1 = OR, 2 = AND, 3 = EOR, 4 = invert
 * @param colour logival colour to use
 */
extern void GCOL(uint8_t action, uint8_t colour);

/**
 * INKEY with a negative number enables you to test whether a particular key
 * is pressed at this instant
 * 
 * @oaram key negative key code
 * @returns 1 if pressed, 0 if not
 */ 
extern uint8_t INKEYNEG(int8_t key);

/**
 * Select which display mode the computer is about to use.
 *
 * @param mode number of new mode
 */
extern void MODE(uint8_t mode);

/**
 * moves the graphics cursor to a particular absolute position without drawing a line
 *
 * @param x new x coordinate in osunits
 * @param y new y coordinate in osunits
 */
static inline void
MOVE(uint16_t x, uint16_t y)
{
	PLOT(4, x, y);
}

/**
 * Text output routine
 * 
 * Note this is sumplified from the BASIC version and can only output
 * single zero terminated strings.
 *
 * @param string string to output (zero terminated)
 */
static inline void
PRINT(const char *string)
{
	puts(string);
}

/**
 * Text output routine at a specific text coordinate
 * 
 * Note this is sumplified from the BASIC version and can only output
 * single zero terminated strings.
 *
 * @param x text coordinate
 * @param y text coordinate
 * @param string string to output (zero terminated)
 */
extern void PRINTTAB(uint8_t x, uint8_t y, const char *string);

/**
 * Generate sounds 
 *
 * @param channel 0-4, 0 = noise channel, 1-3 squarewave
 * @param amplitude loudness -15 (loudest) tp 0 (silence)
 * @param pitch 0-255 pitch, A440 = 89, 4 units per semi-tone, 48 per octave
 * @param duration -1 to 254 note duration in 20ths of a second
 */
extern void SOUND(int16_t channel, int16_t amplitude, int16_t pitch, int16_t duration);

#endif /* BASIC_____H */
